﻿using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Newtonsoft.Json;
using RecrownedAthenaeum.Data;
using RecrownedAthenaeum.SpecialTypes;
using System.IO;

namespace RecrownedAthenaeum.ContentReaders
{
    class TextureAtlasDataReader : ContentTypeReader<TextureAtlas>
    {
        protected override TextureAtlas Read(ContentReader input, TextureAtlas existingInstance)
        {
            string serialized = input.ReadString();
            TextureAtlasData atlasData = JsonConvert.DeserializeObject<TextureAtlasData>(serialized);
            Texture2D atlasTexture = input.ContentManager.Load<Texture2D>(Path.GetFileNameWithoutExtension(atlasData.textureName));
            TextureAtlas atlas = new TextureAtlas(atlasTexture, GenerateAtlasRegionsFromData(atlasData, atlasTexture));

            return atlas;
        }

        /// <summary>
        /// Generates region given <see cref="TextureAtlasData"/>.
        /// </summary>
        /// <param name="textureAtlasData">The data to generate the regions from.</param>
        /// <param name="atlasTexture">The texture containing the atlas.</param>
        /// <returns>An array of regions.</returns>
        public TextureAtlas.Region[] GenerateAtlasRegionsFromData(TextureAtlasData textureAtlasData,  Texture2D atlasTexture)
        {
            TextureAtlas.Region[] regions = new TextureAtlas.Region[textureAtlasData.regions.Length];
            for (int regionID = 0; regionID < regions.Length; regionID++)
            {
                TextureAtlasData.AtlasRegionData regionData = textureAtlasData.regions[regionID];
                NinePatch nPatch = null;
                if (regionData.ninePatchData != null)
                {
                    NinePatchData nPatchData = regionData.ninePatchData;
                    nPatch = new NinePatch(atlasTexture, nPatchData.left, nPatchData.right, nPatchData.bottom, nPatchData.bottom);
                }

                regions[regionID] = new TextureAtlas.Region(regionData.name, regionData.location, nPatch, atlasTexture);
            }

            return regions;
        }
    }
}
